############################################################################
# Unified Plone installer instance build script
# by Steve McMahon (steve at dcn.org)
#
# This script is meant to be invoked by another process
# that will choose the exec Python.
#
# $LastChangedDate: 2008-09-17 13:41:32 -0700 (Wed, 17 Sep 2008) $ $LastChangedRevision: 22584 $

import sys, os, os.path, stat, subprocess, shutil, re

# pick up globals from command line
(UIDIR,
PLONE_HOME,
INSTANCE_HOME,
CLIENT_USER,
ZEO_USER,
PASSWORD,
ROOT_INSTALL,
MYZOPE,
RUN_BUILDOUT,
ITYPE) = sys.argv[1:]


def doCommand(command):
    po = subprocess.Popen(command,
                          shell=True,
                          universal_newlines=True)
    po.communicate()
    return po.returncode


print "Copying standalone buildout template"
shutil.copytree(os.path.join(UIDIR, 'base_skeleton'), INSTANCE_HOME)


# remove OS X and svn detritus (this is mainly helpful for installer development)
doCommand('find %s -name "._*" -exec rm {} \; > /dev/null' % INSTANCE_HOME)
doCommand('find %s -name ".svn" | xargs rm -rf' % INSTANCE_HOME)


######################
# buildout.cfg patches

# read appropriate buildout
fd = open(os.path.join(UIDIR, 'buildout_templates', '%s.cfg' % ITYPE))
buildout = fd.read()
fd.close()

# patch password
# and buildout cache location
buildout = buildout.replace("__PASSWORD__", PASSWORD)
buildout = buildout.replace("__TARGET__", PLONE_HOME)

# Set effective-user
if ROOT_INSTALL == '1':
    buildout = buildout.replace("#effective-user = __CLIENT_USER__", "effective-user = %s" % CLIENT_USER)
    buildout = buildout.replace("#effective-user = __ZEO_USER__", "effective-user = %s" % ZEO_USER)
    buildout = buildout.replace("sudo-command = ", "sudo-command = sudo")
else:
    buildout = re.compile(r"^\s+find .*? chown .*?\n", re.MULTILINE).sub("", buildout)
    buildout = re.compile(r"^\s+chown .*?\n", re.MULTILINE).sub("", buildout)

if MYZOPE != '0':
    # patch buildout.cfg to use the separate zope
    buildout = re.compile(r"^\s+zope2\n", re.MULTILINE).sub("", buildout)
    buildout = re.compile(r"^[zope2]\n", re.MULTILINE).sub("", buildout)
    buildout = buildout.replace("${zope2:location}", MYZOPE)
    buildout = re.compile(r"^recipe = plone.recipe.zope2install", re.MULTILINE).sub("#recipe = plone.recipe.zope2install", buildout)
    buildout = buildout.replace("url = ${plone:zope2-url}\n", "")

fn = os.path.join(INSTANCE_HOME, 'buildout.cfg')
fd = open(fn, 'w')
fd.write(buildout)
fd.close()
os.chmod(fn, stat.S_IRUSR | stat.S_IWUSR)


##########################
# patch bin/buildout paths
fn = os.path.join(INSTANCE_HOME, 'bin', 'buildout')
fd = open(fn)
bbin = fd.read()
fd.close
bbin = bbin.replace('__TARGET__', PLONE_HOME)
fd = open(fn, 'w')
fd.write(bbin)
fd.close()
os.chmod(fn, stat.S_IRWXU)


################
# Start the fun!
if RUN_BUILDOUT == '1':
    print "Running buildout"
    
    os.chdir(INSTANCE_HOME)
    
    returncode = doCommand(os.path.join(INSTANCE_HOME, 'bin', 'buildout') + " -No")
    if returncode:
        print "Buildout returned an error code: %s; Aborting." % returncode
        sys.exit(returncode)

    if ITYPE == 'standalone':
        if not ( os.path.exists(os.path.join(INSTANCE_HOME, 'bin', 'instance')) and
                 os.path.exists(os.path.join(INSTANCE_HOME, 'parts', 'plone')) and
                 os.path.exists(os.path.join(INSTANCE_HOME, 'var')) ):
            print "Parts of the install are missing. Buildout must have failed. Aborting."
            sys.exit(1)
    else:
        if not ( os.path.exists(os.path.join(INSTANCE_HOME, 'bin', 'zeoserver')) and
                 os.path.exists(os.path.join(INSTANCE_HOME, 'bin', 'client1')) and
                 os.path.exists(os.path.join(INSTANCE_HOME, 'parts', 'plone')) and
                 os.path.exists(os.path.join(INSTANCE_HOME, 'var')) ):
            print "Parts of the install are missing. Buildout must have failed. Aborting."
            sys.exit(1)
    

    if ITYPE == 'standalone':
        ###############################
        # Create a Plone instance at /Plone in the ZODB.
        # We don't do this for ZEO installs because we don't
        # want to connect to a server port without giving the user
        # an opportunity to change it.
        print "Creating Plone site..."
        if doCommand("./bin/plonectl init"):
            print "Attempt to create a Plone site in the Zope instance failed."
            print "You will need to do this manually."
        else:
            # successful. cleanup.
            try:
                os.remove(os.path.join(INSTANCE_HOME, 'var', 'filestorage', 'Data.fs.index'))
                os.remove(os.path.join(INSTANCE_HOME, 'var', 'filestorage', 'Data.fs.tmp'))
                os.remove(os.path.join(INSTANCE_HOME, 'var', 'filestorage', 'Data.fs.lock'))
            except OSError:
                pass
else:
    print "Skipping bin/buildout at your request."
